#include <iostream.h>

#include "orxonox_gui.h"
#include "orxonox_gui_video.h"
#include "orxonox_gui_audio.h"


int main( int argc, char *argv[] )
{
  OrxonoxGui* orxonoxgui = new OrxonoxGui(argc, argv);
  return 0;
}

/* ORXONOXGUI */

OrxonoxGui::OrxonoxGui (int argc, char *argv[])
{
  /**
   * Initializes the Gui
   */
  gtk_init (&argc, &argv);

  Window* orxonoxGUI = new Window("Graphical Orxonox Launcher");
  orxonoxGUI->connectSignal ("destroy", orxonoxGUI->orxonox_gui_quit);
  orxonoxGUI->connectSignal ("delete_event", orxonoxGUI->orxonox_gui_quit);
  
  Box* windowBox = new Box ('v');
  {
    Box* avBox = new Box ('h');
    {
      OrxonoxGuiVideo * video = new OrxonoxGuiVideo ();
      avBox->fill (video->getFrame());
      OrxonoxGuiAudio * audio = new OrxonoxGuiAudio ();
      avBox->fill (audio->getFrame());
      
    }
    windowBox->fill (avBox);
  }
  orxonoxGUI->fill (windowBox);
  orxonoxGUI->listOptions();

  /* TEST ENVIRONMENT
  Frame* Frametest = new Frame ("Test");
  orxonoxGUI->fill((Frame*) Frametest);
  Box* box = new Box ('v');
  Frametest->fill(box);
  
  CheckButton* button = new CheckButton("button");
  button->connectSignal ("clicked", orxonoxGUI->orxonox_gui_quit);
  box->fill(button);
  Slider* slider = new Slider("slider", 0, 100);
  slider->connectSignal ("value_changed", slider->OptionChange);
  box->fill(slider);
  Menu* menu = new Menu("menu1", "no output", "verbose", "debug", "lastItem");
  menu->connectSignal ("changed", menu->OptionChange);
  box->fill(menu);
  Menu* menu2 = new Menu("menu2", "no output", "verbose", "debug", "lastItem");
  menu2->connectSignal ("changed", menu->OptionChange);
  box->fill(menu2);
  orxonoxGUI->listOptions();
  */
  orxonoxGUI->showall ();

  
  gtk_main();
}

/* WIDGET */

void Widget::connectSignal (char* event, gint (*signal)(GtkWidget*, GdkEvent*, void *))
{
  /** 
   * Connect any signal to any given Sub-widget
   */
  g_signal_connect (G_OBJECT (this->widget), event, G_CALLBACK (signal), NULL);
}

void Widget::connectSignal (char* event, gint (*signal)( GtkWidget*, Widget *))
{
  /**
   * Connect a signal with additionally passing the whole Object
   */
  g_signal_connect (G_OBJECT (this->widget), event, G_CALLBACK (signal), this);
}

void Widget::show()
{
  /**
   * Function to Show any Widget
   */
  gtk_widget_show (widget);
}

void Widget::listOptions ()
{
  /** 
   * This is For listing all the Options witch are located beneath this Widget.
   */
  if (this->is_option >= 1)
    cout << static_cast<Option*>(this)->option_name <<" is : " << static_cast<Option*>(this)->value <<endl;

  if (this->next != NULL)
    this->next->listOptions ();

  switch (this->is_option)
    {
    case -1:
      static_cast<Container*>(this)->down->listOptions ();
      break;
    case -2:
      static_cast<Box*>(this)->down->listOptions ();
      break;
    } 
}

/* CONTAINERS*/

void Container::fill (Widget *lowerWidget)
{
  /**
   * fill any given Container with a lowerwidet
   */
  if (this->down == NULL)
    {
      gtk_container_add (GTK_CONTAINER (this->widget), lowerWidget->widget);
      this->down = lowerWidget;
    }
  else
    cout << "!!error!! You try to put more than one Widget into a container.\nnot including this item."<<endl;
}

// gtk_container_set_border_width (GTK_CONTAINER (widget), 5);

/* WINDOW */

Window::Window (void)
{
  /**
   * Creating a Window
   */
  is_option = -1;
  widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy (GTK_WINDOW(widget), TRUE, TRUE, TRUE);
  next = NULL;
  down = NULL;
}

Window::Window (char* windowName)
{
  /**
   * Creating a Window with passing a name
   */
  is_option = -1;
  widget = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy (GTK_WINDOW (widget), TRUE, TRUE, TRUE);
  this->setTitle (windowName);
}

Window::~Window ()
{
  /**
   * Destoying a Window (not implemented)
   */
}

void Window::showall ()
{
  /**
   * Shows all Widgets that are included within this Widget
   */
  gtk_widget_show_all  (widget);
}

void Window::setTitle (char* title)
{
  /**
   * Set The Window title to title
   */
  gtk_window_set_title (GTK_WINDOW (widget), title);
}


gint Window::orxonox_gui_quit (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  /**
   * Quits the orxonox_GUI
   */
  gtk_main_quit();
  return FALSE;
}


/* FRAME */

Frame::Frame (void)
{
  /** 
   * Creates a new Frame without a name
   */
  is_option = -1;
  next = NULL;
  down = NULL;
  widget = gtk_frame_new ("");
}
Frame::Frame (char* title)
{
  /**
   * Creates a new Frame with name title
   */
  is_option = -1;
  next = NULL;
  down = NULL;
  widget = gtk_frame_new (title);
}
Frame::~Frame ()
{
  /**
   * Destroys given Frame (not implemented yet)
   */
}

void Frame::setTitle (char* title)
{
  /**
   * Sets the Frames name to title
   */
  gtk_frame_set_label (GTK_FRAME (widget), title);
}



/* BOX */

Box::Box (void)
{
  /**
   * Creates a new horizontal Box
   */
  is_option = -2;
  next = NULL;
  down = NULL;
  widget = gtk_hbox_new(FALSE, 0);
}
Box::Box (char boxtype)
{
  /**
   * Creates a new Box if boxtype is 'v' it will be vertically, if 'h' it will be horizontally
   */
  is_option = -2;
  next = NULL;
  down = NULL;
  if (boxtype == 'v')
    {
      widget = gtk_vbox_new (FALSE, 0);
    }
  else
    {
      widget = gtk_hbox_new (FALSE, 0);
    }
}

Box::~Box ()
{
  /**
   * Destroys given Frame (not implemented yet)
   */
}

void Box::fill (Widget *lowerWidget)
{
  /**
   * Fill a Box with its lowerwidgets
   */
  gtk_box_pack_start (GTK_BOX (this->widget), lowerWidget->widget, TRUE, TRUE, 0);
  if (this->down == NULL)
    this->down = lowerWidget;
  else
    {
      Widget* tmp;
      tmp = this->down;
      while (tmp->next != NULL)
	{
	  tmp = tmp->next;
	}
      tmp->next = lowerWidget;
    }
}


/* OPTION */

/* BUTTON */
Button::Button(char* buttonname)
{
  /**
   * Creates a new Button with name buttonname
   */
  is_option = 0;
  value = 0;
  next = NULL;
  option_name = buttonname;
  widget = gtk_button_new_with_label (buttonname);
}

/* CHECKBUTTON */
CheckButton::CheckButton (char* buttonname)
{
  /**
   * Creates a new CheckButton with name buttonname
   */
  is_option = 1;
  value = 0;
  next = NULL;
  option_name = buttonname;
  widget = gtk_check_button_new_with_label (buttonname);

  this->connectSignal ("clicked", this->OptionChange);
}
gint CheckButton::OptionChange (GtkWidget *widget, Widget* checkbutton)
{
  /** 
   * Writes value, if changed on the checkbutton, to the object.
   */
  static_cast<CheckButton*>(checkbutton)->value = (int)gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON ((CheckButton*)checkbutton->widget));
  cout << static_cast<CheckButton*>(checkbutton)->option_name << " set to: " << static_cast<CheckButton*>(checkbutton)->value << endl;
}

/* SLIDER */
Slider::Slider (char* slidername, int start, int end)
{
  /**
   * Creates a new Slider with name slidername a beginning value of start and an end value of end.
   */
  is_option = 1;
  value = 0;
  next = NULL;
  option_name = slidername;
  widget = gtk_hscale_new_with_range (start, end, 5);
  value = start;

  this->connectSignal ("value_changed", this->OptionChange);
}

gint Slider::OptionChange (GtkWidget *widget, Widget* slider)
{
  /** 
   * Writes value, if changed on the slider, to the object.
   */
  static_cast<Slider*>(slider)->value = (int)gtk_range_get_value (GTK_RANGE ((Slider*)slider->widget));
  cout << static_cast<Slider*>(slider)->option_name << " set to: "<< static_cast<Slider*>(slider)->value << endl;
}


Menu::Menu (char* menuname, ...)
{
  /** 
   * Creates an Menu-Item-list out of multiple input. Consider, that the last input argument has to be "lastItem" for this to work.
   */
  is_option = 1;
  value = 0;
  next = NULL;
  option_name = menuname;
  char *tmp;
  va_list itemlist;
  widget = gtk_option_menu_new ();
  GtkWidget* menu = gtk_menu_new ();
  GtkWidget* item;

  va_start (itemlist, menuname);
  while (strcmp (tmp = va_arg (itemlist, char*), "lastItem"))
    {
      item = gtk_menu_item_new_with_label (tmp);
      gtk_menu_shell_append(GTK_MENU_SHELL (menu), item);
    }
  va_end(itemlist);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (widget), menu);
}

gint Menu::OptionChange (GtkWidget *widget, Widget* menu)
{
  /** 
   * Writes value, if changed on the Menu, to the object.
   */
  static_cast<Menu*>(menu)->value = (int)gtk_option_menu_get_history (GTK_OPTION_MENU (menu->widget));
  cout << static_cast<Menu*>(menu)->option_name << " changed to : " << static_cast<Menu*>(menu)->value << endl;
}
